const { app, ipcMain, Menu, BrowserWindow } = require('electron');
const path = require('path');
const url = require('url');
const express = require("express");
const cookieParser = require('cookie-parser');
const ajax = require("najax");


let server = express();
server.use(cookieParser());

var AuthenticationContext = require('adal-node').AuthenticationContext;

var clientId = '...';
var clientSecret = '...'
var authorityHostUrl = 'https://login.windows.net';
var tenant = 'foo.onmicrosoft.com';
var authorityUrl = authorityHostUrl + '/' + tenant;
var redirectUri = 'http://localhost:3000/getAToken';
var resource = 'https://foo.sharepoint.com';
var templateAuthzUrl = 'https://login.windows.net/' +
    tenant +
    '/oauth2/authorize?response_type=code&client_id=' +
    clientId +
    '&redirect_uri=' +
    redirectUri +
    '&state=<state>&resource=' +
    resource;

var authenticationContext = new AuthenticationContext(authorityUrl);

var tokenResponse = {};

function createAuthorizationUrl(state) {
    return templateAuthzUrl.replace('<state>', state);
}
server.get('/auth', function (req, res) {
    var token = "foo-token";
    res.cookie('authstate', token);
    var authorizationUrl = createAuthorizationUrl(token);
    res.redirect(authorizationUrl);
});
server.get('/getAToken', function (req, res) {
    if (req.cookies.authstate !== req.query.state) {
        res.send('error: state does not match');
    }
    authenticationContext.acquireTokenWithAuthorizationCode(
        req.query.code,
        redirectUri,
        resource,
        clientId,
        clientSecret,
        function (err, response) {
            var message = '';
            if (err) {
                message = 'error: ' + err.message + '\n';
                res.status(400).send(message);
            } else {
                tokenResponse = response;
                win.webContents.send("loginResponse",tokenResponse);
                res.status(200).send("Logged in");
            }            
        }
    );
});
server.listen(3000);

let win;

function createWindow() {
    win = new BrowserWindow({
        width: 800,
        height: 600
    });
    win.loadURL(url.format({
        pathname: path.join(__dirname, 'index.html'),
        protocol: 'file:',
        slashes: true
    }));

    // Open the DevTools.
    win.webContents.openDevTools();

    win.on('closed', () => {
        win = null;
    });
}

app.on('ready', createWindow);

// Quit when all windows are closed.
app.on('window-all-closed', () => {
    if (process.platform !== 'darwin') {
        app.quit();
    }
});

app.on('activate', () => {
    if (win === null) {
        createWindow();
    }
});

ipcMain.on("message", (event, argument) => {    
    ajax({
        url: resource+"/_api/web/lists",
        "method": "GET",
        "contentType": "application/json;odata=verbose",
        "headers": {
            "Accept": "application/json;odata=verbose",
            "Authorization": "Bearer " + tokenResponse.accessToken,
            'Access-Control-Allow-Origin': '*'
        },
        "crossDomain": true,
        "async": true,
        "cache": false,
        "processData": false
    }).success(function (data) {
        console.log(JSON.stringify(data));
        event.returnValue = JSON.stringify(data);
    }).error(function (data) {
        console.log(JSON.stringify(data));
        event.returnValue = JSON.stringify(data);
    });
});